/* GENERATED FILE */
/* use ./gendef0 to regenerate this file */
#ifndef ENOSYS
#define ENOSYS 0
#endif
#ifndef EXDEV
#define EXDEV 0
#endif
#ifndef EBADR
#define EBADR 0
#endif
#ifndef O_RDONLY
#define O_RDONLY 0
#endif
#ifndef O_WRONLY
#define O_WRONLY 0
#endif
#ifndef O_RDWR
#define O_RDWR 0
#endif
#ifndef O_CREAT
#define O_CREAT 0
#endif
#ifndef O_TRUNC
#define O_TRUNC 0
#endif
#ifndef O_EXCL
#define O_EXCL 0
#endif
#ifndef O_APPEND
#define O_APPEND 0
#endif
#ifndef O_ASYNC
#define O_ASYNC 0
#endif
#ifndef O_DIRECT
#define O_DIRECT 0
#endif
#ifndef O_NOATIME
#define O_NOATIME 0
#endif
#ifndef O_CLOEXEC
#define O_CLOEXEC 0
#endif
#ifndef O_NOCTTY
#define O_NOCTTY 0
#endif
#ifndef O_NOFOLLOW
#define O_NOFOLLOW 0
#endif
#ifndef O_NONBLOCK
#define O_NONBLOCK 0
#endif
#ifndef O_EXEC
#define O_EXEC 0
#endif
#ifndef O_SEARCH
#define O_SEARCH 0
#endif
#ifndef O_DIRECTORY
#define O_DIRECTORY 0
#endif
#ifndef O_DSYNC
#define O_DSYNC 0
#endif
#ifndef O_RSYNC
#define O_RSYNC 0
#endif
#ifndef O_SYNC
#define O_SYNC 0
#endif
#ifndef O_TTY_INIT
#define O_TTY_INIT 0
#endif
#ifndef S_IFIFO
#define S_IFIFO 0
#endif
#ifndef S_IFCHR
#define S_IFCHR 0
#endif
#ifndef S_IFBLK
#define S_IFBLK 0
#endif
#ifndef S_IFLNK
#define S_IFLNK 0
#endif
#ifndef S_IFREG
#define S_IFREG 0
#endif
#ifndef S_IFDIR
#define S_IFDIR 0
#endif
#ifndef S_IFWHT
#define S_IFWHT 0
#endif
#ifndef S_IFSOCK
#define S_IFSOCK 0
#endif
#ifndef S_IFMT
#define S_IFMT 0
#endif
#ifndef POSIX_FADV_NORMAL
#define POSIX_FADV_NORMAL 0
#endif
#ifndef POSIX_FADV_SEQUENTIAL
#define POSIX_FADV_SEQUENTIAL 0
#endif
#ifndef POSIX_FADV_RANDOM
#define POSIX_FADV_RANDOM 0
#endif
#ifndef POSIX_FADV_NOREUSE
#define POSIX_FADV_NOREUSE 0
#endif
#ifndef POSIX_FADV_WILLNEED
#define POSIX_FADV_WILLNEED 0
#endif
#ifndef POSIX_FADV_DONTNEED
#define POSIX_FADV_DONTNEED 0
#endif
#ifndef POSIX_MADV_NORMAL
#define POSIX_MADV_NORMAL 0
#endif
#ifndef POSIX_MADV_SEQUENTIAL
#define POSIX_MADV_SEQUENTIAL 0
#endif
#ifndef POSIX_MADV_RANDOM
#define POSIX_MADV_RANDOM 0
#endif
#ifndef POSIX_MADV_WILLNEED
#define POSIX_MADV_WILLNEED 0
#endif
#ifndef POSIX_MADV_DONTNEED
#define POSIX_MADV_DONTNEED 0
#endif
#ifndef ST_RDONLY
#define ST_RDONLY 0
#endif
#ifndef ST_NOSUID
#define ST_NOSUID 0
#endif
#ifndef ST_NODEV
#define ST_NODEV 0
#endif
#ifndef ST_NOEXEC
#define ST_NOEXEC 0
#endif
#ifndef ST_SYNCHRONOUS
#define ST_SYNCHRONOUS 0
#endif
#ifndef ST_MANDLOCK
#define ST_MANDLOCK 0
#endif
#ifndef ST_WRITE
#define ST_WRITE 0
#endif
#ifndef ST_APPEND
#define ST_APPEND 0
#endif
#ifndef ST_IMMUTABLE
#define ST_IMMUTABLE 0
#endif
#ifndef ST_NOATIME
#define ST_NOATIME 0
#endif
#ifndef ST_NODIRATIME
#define ST_NODIRATIME 0
#endif
#ifndef ST_RELATIME
#define ST_RELATIME 0
#endif
#ifndef PROT_NONE
#define PROT_NONE 0
#endif
#ifndef PROT_EXEC
#define PROT_EXEC 0
#endif
#ifndef PROT_READ
#define PROT_READ 0
#endif
#ifndef PROT_WRITE
#define PROT_WRITE 0
#endif
#ifndef MAP_PRIVATE
#define MAP_PRIVATE 0
#endif
#ifndef MAP_SHARED
#define MAP_SHARED 0
#endif
#ifndef MAP_ANONYMOUS
#define MAP_ANONYMOUS 0
#endif
#ifndef MAP_HUGETLB
#define MAP_HUGETLB 0
#endif
#ifndef MAP_LOCKED
#define MAP_LOCKED 0
#endif
#ifndef MAP_NORESERVE
#define MAP_NORESERVE 0
#endif
#ifndef MAP_POPULATE
#define MAP_POPULATE 0
#endif
#ifndef MAP_NONBLOCK
#define MAP_NONBLOCK 0
#endif
#ifndef FIEMAP_FLAG_SYNC
#define FIEMAP_FLAG_SYNC 0
#endif
#ifndef FIEMAP_FLAG_XATTR
#define FIEMAP_FLAG_XATTR 0
#endif
#ifndef FIEMAP_FLAGS_COMPAT
#define FIEMAP_FLAGS_COMPAT 0
#endif
#ifndef FIEMAP_EXTENT_LAST
#define FIEMAP_EXTENT_LAST 0
#endif
#ifndef FIEMAP_EXTENT_UNKNOWN
#define FIEMAP_EXTENT_UNKNOWN 0
#endif
#ifndef FIEMAP_EXTENT_DELALLOC
#define FIEMAP_EXTENT_DELALLOC 0
#endif
#ifndef FIEMAP_EXTENT_ENCODED
#define FIEMAP_EXTENT_ENCODED 0
#endif
#ifndef FIEMAP_EXTENT_DATA_ENCRYPTED
#define FIEMAP_EXTENT_DATA_ENCRYPTED 0
#endif
#ifndef FIEMAP_EXTENT_NOT_ALIGNED
#define FIEMAP_EXTENT_NOT_ALIGNED 0
#endif
#ifndef FIEMAP_EXTENT_DATA_INLINE
#define FIEMAP_EXTENT_DATA_INLINE 0
#endif
#ifndef FIEMAP_EXTENT_DATA_TAIL
#define FIEMAP_EXTENT_DATA_TAIL 0
#endif
#ifndef FIEMAP_EXTENT_UNWRITTEN
#define FIEMAP_EXTENT_UNWRITTEN 0
#endif
#ifndef FIEMAP_EXTENT_MERGED
#define FIEMAP_EXTENT_MERGED 0
#endif
#ifndef FIEMAP_EXTENT_SHARED
#define FIEMAP_EXTENT_SHARED 0
#endif
#ifndef SPLICE_F_MOVE
#define SPLICE_F_MOVE 0
#endif
#ifndef SPLICE_F_NONBLOCK
#define SPLICE_F_NONBLOCK 0
#endif
#ifndef SPLICE_F_MORE
#define SPLICE_F_MORE 0
#endif
#ifndef SPLICE_F_GIFT
#define SPLICE_F_GIFT 0
#endif
#ifndef SEEK_DATA
#define SEEK_DATA 0
#endif
#ifndef SEEK_HOLE
#define SEEK_HOLE 0
#endif
